module Data.Graph.Infomap.Internal where

import Control.Exception
import Foreign.C.String
import Foreign.C.Types
import Foreign.Ptr

newtype Network = Network (Ptr Network)

foreign import ccall unsafe "network_create" createNetwork
  :: CString -> IO Network

foreign import ccall unsafe "network_destroy" destroyNetwork
  :: Network -> IO ()

foreign import ccall unsafe "add_link" addLink
  :: Network -> CUInt -> CUInt -> IO ()

foreign import ccall unsafe "run" run
  :: Network -> IO ()

foreign import ccall unsafe "getCodelength" getCodelength
  :: Network -> IO CDouble

type NetworkCfg = String

withNetwork :: NetworkCfg -> (Network -> IO a) -> IO a
withNetwork cfgstr f = withCString cfgstr $ \cfgcstr ->
  bracket (createNetwork cfgcstr) destroyNetwork f

test :: IO ()
test = withNetwork "--two-level -N2" $ \n -> do
  addLink n 0 1
  addLink n 0 2
  addLink n 0 3
  addLink n 1 0
  addLink n 1 2
  addLink n 2 1
  addLink n 2 0
  addLink n 3 0
  addLink n 3 4
  addLink n 3 5
  addLink n 4 3
  addLink n 4 5
  addLink n 5 4
  addLink n 5 3
  run n
  codelen <- getCodelength n
  putStrLn $ "codelength: " ++ show codelen
